﻿// CleanPages.jsx
// An InDesign JavaScript
//
// Cleaning the pages of active document
//
// Copyright: Sho Nuff - shonuff@ya.ru
// License: MIT
//
// 2012-04-24

if (!app.documents.length) exit(); // нет открытых документов

// все глобальные переменные
var 
	aDoc =      app.activeDocument,
	// script preferences
	prefsFile = new File(Folder.userData + '/CleanPages.conf' );
	prefs =     {
		startPageNumber:  0,
		endPageNumber:    aDoc.pages.length - 1,
		activeLayers:     '',
		removeTextFrames: true,
		removePolygons:   true,
		removeOther:      true,
		removeLocked:     false
	};

// прочитаем предыдущие настройки
if (prefsFile.exists && prefsFile.open('r')) {
	
	try {
		prefs = eval(prefsFile.read());
	} catch (e) {}
	
	prefsFile.close();
}

// уточним настройки
if (prefs.startPageNumber > aDoc.pages.length - 1) prefs.startPageNumber = aDoc.pages.length - 1;
if (prefs.endPageNumber > aDoc.pages.length - 1) prefs.endPageNumber = aDoc.pages.length - 1;

// создаем диалог
(function() {
	var 
		pageNames =  [],
		layerNames = [];
	
	// собираем в массив имена страниц
	// если нужно дбавляем префикс
	for (var i = 0, l = aDoc.pages.length; i < l; i++) {
		var 
			page =    aDoc.pages[i],
			section = page.appliedSection;
		
		pageNames.push(
			(section.includeSectionPrefix ? '' :  section.sectionPrefix) +
			page.name);
	}

	// собираем в массив названия слоев
	for (var i = 0, l = aDoc.layers.length; i < l; i++) {
		layerNames.push(aDoc.layers[i].name);
	}
	var dialog = app.dialogs.add({name: 'CleanPages'});
	var col1 = dialog.dialogColumns.add();

	// выбор первой страницы диапазона
	var rowStartPage = col1.dialogRows.add();
	rowStartPage.staticTexts.add({staticLabel: localize({en: 'Start Page', ru: 'Начать со страницы:'}), minWidth: 150});
	var ddStartPage = rowStartPage.dropdowns.add({stringList: pageNames, selectedIndex: prefs.startPageNumber, minWidth: 100});

	// выбор последней страницы диапазона
	var rowEndPage = col1.dialogRows.add();
	rowEndPage.staticTexts.add({staticLabel: localize({en: 'End Page:', ru: 'Закончить страницей:'}), minWidth: 150});
	var ddEndPage = rowEndPage.dropdowns.add({stringList: pageNames, selectedIndex: prefs.endPageNumber, minWidth: 100});

	// список слоев
	var colLayer = col1.dialogRows.add().borderPanels.add().dialogColumns.add();
	colLayer.dialogRows.add().staticTexts.add({staticLabel: localize({
			en: 'Clean Layers:                                   ', 
			ru: 'Очистить слои:                                  '})});
	var cbLayers = [];

	for (var i = 0, l = layerNames.length; i < l; i++) {
		var layerName = layerNames[i];
		
		cbLayers[i] = colLayer.dialogRows.add().checkboxControls.add({
			staticLabel: layerName, 
			checkedState: prefs.activeLayers.indexOf('\t' + layerName + '\t') >= 0
		});
	}
	
	// список объектов
	var colObjects = col1.dialogRows.add().borderPanels.add().dialogColumns.add();
	colObjects.dialogRows.add().staticTexts.add({staticLabel: localize({
			en:'Remove:                                        ',
			ru: 'Удалять объекты:                              '})});
	
	var cbTextFrames = colObjects.dialogRows.add().checkboxControls.add({
		staticLabel: localize({en: 'Text Frames', ru: 'Текстовые фреймы'}), 
		checkedState: !!prefs.removeTextFrames
	});
	var cbPolygons = colObjects.dialogRows.add().checkboxControls.add({
		staticLabel: localize({en: 'Lines, Ovals, Polygons', ru: 'Линии, овалы, многоугольники'}), 
		checkedState: !!prefs.removePolygons
	});
	var cbOther = colObjects.dialogRows.add().checkboxControls.add({
		staticLabel: localize({en:'Other', ru: 'Группы, кнопки и прочие'}), 
		checkedState: !!prefs.removeOther
	});

	// очистка залоченых
	var cbLocked = col1.dialogRows.add().checkboxControls.add({
		staticLabel: localize({en: 'Remove Locked', ru: 'Очищать заблокированные'}), 
		checkedState: prefs.removeLocked
	});
	
	while (true) {
		// покажем диалог
		if (!dialog.show()) {
			// если нажата кнопка отмены - выход
			dialog.destroy(); 
			exit();
		}
	
		// обновим prefs
		prefs.startPageNumber = Math.min(ddStartPage.selectedIndex, ddEndPage.selectedIndex);
		prefs.endPageNumber =   Math.max(ddStartPage.selectedIndex, ddEndPage.selectedIndex);
		var str = '\t';
		
		for (var i = cbLayers.length; i--; ) {
			if (!cbLayers[i].checkedState) continue;
			str += layerNames[i] + '\t';
		}
		if (str.length < 2) {
			alert(localize({en: 'Select Layer, please', ru: 'Нужно указать хотя бы один слой'}));
			continue;
		}
		prefs.activeLayers = str;
		
		if (!(cbTextFrames.checkedState || 
			cbPolygons.checkedState || cbOther.checkedState)) {
			
			alert(localize({en: 'Choice Type of Objects', ru: 'Не выбраны объекты для удаления'}));
			continue;
		}
		prefs.removeTextFrames = cbTextFrames.checkedState;
		prefs.removePolygons =   cbPolygons.checkedState;
		prefs.removeOther =      cbOther.checkedState;
		prefs.removeLocked =     cbLocked.checkedState;
		
		break;
	}
	dialog.destroy();
})();
// конец диалога

// сохраним prefs
if (!prefsFile.open('w') || !prefsFile.write(prefs.toSource())) {
	alert(localise({en: 'Error. Can not save settings', ru: 'Ошибка\nНе удалось сохранить настройки.'}));
}
prefsFile.close();

// обходит коллекцию pageItems и удаляет подходящие по параметрам элементы
function CleanPageItems(pageItems) {
	seekPageItems:
	for (var i = pageItems.length; i--; ) {
		try {
			var pageItem = pageItems[i];
		}catch (e) {
			continue;
		}
		// пропустим залоченые
		if (pageItem.locked && !prefs.removeLocked) continue;

		// проверим слой
		if (prefs.activeLayers.indexOf('\t' + pageItem.itemLayer.name + '\t') < 0) continue;
		if (pageItem.itemLayer.locked) continue;
		
		// проверим конструктор
		switch (pageItem.constructor.name) {
			case 'TextFrame':
				if (!prefs.removeTextFrames) {
					continue seekPageItems;
				}
				break;
			
			case 'GraphicLine':
			case 'Oval':
			case 'Polygon':
			case 'Rectangle':
				if (!prefs.removePolygons) continue seekPageItems;
				break;
			
			default: 
				if (!prefs.removeOther) continue seekPageItems;
		}
		pageItem.locked = false;
		pageItem.remove();
	}
}

// обернем основные операции в одну функцию 
// чтобы все действия скрипта можно было отменить одной командой
function CleanPages() {
	var delockedLayers = [];
		
	// разлочим все слои
	if (prefs.removeLocked) {
		for (var i = aDoc.layers.length; i--; ) {
			var layer = aDoc.layers[i];
			
			if (layer.locked) {
				delockedLayers.push(layer);
				layer.locked = false;
			}
		}
	}

	// обход страниц
	for (var i = prefs.startPageNumber; i <= prefs.endPageNumber; i++) {
		var page = aDoc.pages[i];
		CleanPageItems(page.pageItems);
	}

	// залочим обратно слои
	for (var i = delockedLayers.length; i--; ) {
		delockedLayers[i].locked = true;
	}
}

if (parseFloat(app.version.split('.')[0]) < 5) {
	// в старых версиях ИД просто вызываем функцию
	CleanPages();
} else {
	// в ТД 5 и старше cгруппируем все операции в одну
	app.doScript(CleanPages, undefined, undefined, UndoModes.FAST_ENTIRE_SCRIPT, 'Clean Pages');
}
